@@ -21,6 +21,7 @@ create_dialog (void)
 
 	if (dialog) {
 		GtkWidget *image;
+		gchar *prog;
 
 		image = gtk_image_new_from_stock (GTK_STOCK_QUIT, GTK_ICON_SIZE_BUTTON);
 		gtk_button_set_image (GTK_BUTTON (WID ("at_close_logout_button")), image);
@@ -31,6 +32,9 @@ create_dialog (void)
 		image = gtk_image_new_from_stock (GTK_STOCK_JUMP_TO, GTK_ICON_SIZE_BUTTON);
 		gtk_button_set_image (GTK_BUTTON (WID ("keyboard_button")), image);
 
+		image = gtk_image_new_from_stock (GTK_STOCK_JUMP_TO, GTK_ICON_SIZE_BUTTON);
+		gtk_button_set_image (GTK_BUTTON (WID ("mouse_button")), image);
+
 		image = gtk_image_new_from_stock (GTK_STOCK_JUMP_TO, GTK_ICON_SIZE_BUTTON);
 		gtk_button_set_image (GTK_BUTTON (WID ("login_button")), image);
 
@@ -39,6 +43,12 @@ create_dialog (void)
 
 		gtk_image_set_from_file (GTK_IMAGE (WID ("at_applications_image")),
 					 PIXMAPDIR "/at-support.png");
+
+		prog = g_find_program_in_path ("gdmsetup");
+		if (prog == NULL)
+			gtk_widget_hide (WID ("login_button"));
+
+		g_free (prog);
 	}
 
 	return dialog;
@@ -47,19 +57,25 @@ create_dialog (void)
 static void
 cb_at_preferences (GtkDialog *dialog, gint response_id)
 {
-	g_spawn_command_line_async("gnome-default-applications-properties", NULL);
+	g_spawn_command_line_async ("gnome-default-applications-properties --show-page=a11y", NULL);
 }
 
 static void
 cb_keyboard_preferences (GtkDialog *dialog, gint response_id)
 {
-	g_spawn_command_line_async("gnome-accessibility-keyboard-properties", NULL);
+	g_spawn_command_line_async ("gnome-keyboard-properties --a11y", NULL);
+}
+
+static void
+cb_mouse_preferences (GtkDialog *dialog, gint response_id)
+{
+	g_spawn_command_line_async ("gnome-mouse-properties --show-page=accessibility", NULL);
 }
 
 static void
 cb_login_preferences (GtkDialog *dialog, gint response_id)
 {
-	g_spawn_command_line_async("gdmsetup", NULL);
+	g_spawn_command_line_async ("gdmsetup", NULL);
 }
 
 static void
@@ -107,15 +123,13 @@ at_enable_toggled (GtkToggleButton *toggle_button,
 }
 
 static void
-at_enable_update  (GConfClient *client,
-		   GladeXML    *dialog)
+at_enable_update (GConfClient *client,
+		  GladeXML    *dialog)
 {
 	gboolean is_enabled = gconf_client_get_bool (client, ACCESSIBILITY_KEY, NULL);
 
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WID ("at_enable_toggle")),
 				      is_enabled);
-
-	gtk_widget_set_sensitive (WID ("at_pref_button"), is_enabled);
 }
 
 static void
@@ -165,6 +179,10 @@ setup_dialog (GladeXML *dialog)
 			  "clicked",
 			  G_CALLBACK (cb_keyboard_preferences), NULL);
 
+	g_signal_connect (G_OBJECT (WID("mouse_button")),
+			  "clicked",
+			  G_CALLBACK (cb_mouse_preferences), NULL);
+
 	g_signal_connect (G_OBJECT (WID("login_button")),
 			  "clicked",
 			  G_CALLBACK (cb_login_preferences), NULL);
