@@ -42,7 +42,8 @@ enum
 {
   RESPONSE_APPLY_BG,
   RESPONSE_REVERT_FONT,
-  RESPONSE_APPLY_FONT
+  RESPONSE_APPLY_FONT,
+  RESPONSE_INSTALL_ENGINE
 };
 
 enum
@@ -57,6 +58,8 @@ static const GtkTargetEntry drop_types[] =
   {"_NETSCAPE_URL", 0, TARGET_NS_URL}
 };
 
+static void theme_message_area_update (AppearanceData *data);
+
 static time_t
 theme_get_mtime (const char *name)
 {
@@ -65,19 +68,21 @@ theme_get_mtime (const char *name)
 
   theme = gnome_theme_meta_info_find (name);
   if (theme != NULL) {
-    GnomeVFSFileInfo *file_info;
-    GnomeVFSResult result;
+    GFile *file;
+    GFileInfo *file_info;
 
-    file_info = gnome_vfs_file_info_new ();
+    file = g_file_new_for_path (theme->path);
+    file_info = g_file_query_info (file,
+                                   G_FILE_ATTRIBUTE_TIME_MODIFIED,
+                                   G_FILE_QUERY_INFO_NONE,
+                                   NULL, NULL);
+    g_object_unref (file);
 
-    result = gnome_vfs_get_file_info (theme->path, file_info,
-				      GNOME_VFS_FILE_INFO_DEFAULT |
-				      GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
-
-    if (result == GNOME_VFS_OK)
-      mtime = file_info->mtime;
-
-    gnome_vfs_file_info_unref (file_info);
+    if (file_info != NULL) {
+      mtime = g_file_info_get_attribute_uint64 (file_info,
+                                                G_FILE_ATTRIBUTE_TIME_MODIFIED);
+      g_object_unref (file_info);
+    }
   }
 
   return mtime;
@@ -350,67 +355,6 @@ theme_is_equal (const GnomeThemeMetaInfo *a, const GnomeThemeMetaInfo *b)
   return TRUE;
 }
 
-static gchar *
-theme_validate (const GnomeThemeMetaInfo *info)
-{
-  GnomeThemeInfo *theme;
-  gchar *gtkrc;
-
-  theme = gnome_theme_info_find (info->gtk_theme_name);
-  if (!theme || !theme->has_gtk) {
-    return g_strdup_printf (
-        _("This theme will not look as intended because the required GTK+ theme '%s' is not installed."),
-          info->gtk_theme_name);
-  }
-
-  theme = gnome_theme_info_find (info->metacity_theme_name);
-  if (!theme || !theme->has_metacity) {
-    return g_strdup_printf (
-        _("This theme will not look as intended because the required window manager theme '%s' is not installed."),
-          info->metacity_theme_name);
-  }
-
-  if (!gnome_theme_icon_info_find (info->icon_theme_name)) {
-    return g_strdup_printf (
-        _("This theme will not look as intended because the required icon theme '%s' is not installed."),
-        info->gtk_theme_name);
-  }
-
-  /* check for gtk theme engines */
-  gtkrc = gtkrc_find_named (info->gtk_theme_name);
-  if (gtkrc) {
-    GSList *engines = NULL, *l;
-    gchar *msg = NULL;
-    gboolean found;
-
-    gtkrc_get_details (gtkrc, &engines, NULL);
-    g_free (gtkrc);
-
-    for (l = engines; l; l = l->next) {
-      gchar *lib = g_strconcat ("lib", l->data, ".so", NULL);
-      gchar *full = g_build_filename (GTK_ENGINE_DIR, lib, NULL);
-
-      g_free (lib);
-      found = g_file_test (full, G_FILE_TEST_EXISTS);
-      g_free (full);
-
-      if (!found) {
-        msg = g_strdup_printf (
-            _("This theme will not look as intended because the required GTK+ theme engine '%s' is not installed."),
-            (gchar *) l->data);
-        break;
-      }
-    }
-
-    g_slist_foreach (engines, (GFunc) g_free, NULL);
-    g_slist_free (engines);
-
-    return msg;
-  }
-
-  return NULL;
-}
-
 static void
 theme_set_custom_from_theme (const GnomeThemeMetaInfo *info, AppearanceData *data)
 {
@@ -500,6 +444,7 @@ theme_message_area_response_cb (GtkWidget *w,
 {
   const GnomeThemeMetaInfo *theme;
   gchar *tmpfont;
+  gchar *engine_path;
 
   theme = theme_get_selected (GTK_ICON_VIEW (glade_xml_get_widget (data->xml, "theme_list")), data);
   if (!theme)
@@ -625,6 +570,16 @@ theme_message_area_response_cb (GtkWidget *w,
                                  theme->monospace_font, NULL);
       }
       break;
+
+    case RESPONSE_INSTALL_ENGINE:
+      engine_path = gtk_theme_info_missing_engine (theme->gtk_theme_name, FALSE);
+      if (engine_path != NULL) {
+              theme_install_file (GTK_WINDOW (gtk_widget_get_toplevel (data->install_button)),
+                                  engine_path);
+              g_free (engine_path);
+      }
+      theme_message_area_update (data);
+      break;
   }
 }
 
@@ -637,16 +592,18 @@ theme_message_area_update (AppearanceData *data)
   gboolean show_revert_font = FALSE;
   gboolean show_error;
   const gchar *message;
-  gchar *error_message;
   gchar *font;
+  GError *error = NULL;
 
   theme = theme_get_selected (GTK_ICON_VIEW (glade_xml_get_widget (data->xml, "theme_list")), data);
 
-  if (!theme)
+  if (!theme) {
+    if (data->theme_message_area != NULL)
+      gtk_widget_hide (data->theme_message_area);
     return;
+  }
 
-  error_message = theme_validate (theme);
-  show_error = (error_message != NULL);
+  show_error = !gnome_theme_meta_info_validate (theme, &error);
 
   if (!show_error) {
     if (theme->background_image != NULL) {
@@ -723,6 +680,10 @@ theme_message_area_update (AppearanceData *data)
         GEDIT_MESSAGE_AREA (data->theme_message_area),
         _("Revert Font"),
         RESPONSE_REVERT_FONT);
+    data->install_button = gedit_message_area_add_button (
+        GEDIT_MESSAGE_AREA (data->theme_message_area),
+        _("Install"),
+        RESPONSE_INSTALL_ENGINE);
 
     data->theme_message_label = gtk_label_new (NULL);
     gtk_widget_show (data->theme_message_label);
@@ -745,7 +706,7 @@ theme_message_area_update (AppearanceData *data)
   }
 
   if (show_error)
-    message = error_message;
+    message = error->message;
   else if (show_apply_background && show_apply_font && show_revert_font)
     message = _("The current theme suggests a background and a font. Also, the last applied font suggestion can be reverted.");
   else if (show_apply_background && show_revert_font)
@@ -778,6 +739,13 @@ theme_message_area_update (AppearanceData *data)
   else
     gtk_widget_hide (data->revert_font_button);
 
+  if (show_error
+      && g_error_matches (error, GNOME_THEME_ERROR, GNOME_THEME_ERROR_GTK_ENGINE_NOT_AVAILABLE)
+      && packagekit_available ())
+    gtk_widget_show (data->install_button);
+  else
+    gtk_widget_hide (data->install_button);
+
   if (show_error || show_apply_background || show_apply_font || show_revert_font) {
     gtk_widget_show (data->theme_message_area);
     gtk_widget_queue_draw (data->theme_message_area);
@@ -794,7 +762,7 @@ theme_message_area_update (AppearanceData *data)
   }
 
   gtk_label_set_text (GTK_LABEL (data->theme_message_label), message);
-  g_free (error_message);
+  g_clear_error (&error);
 }
 
 static void
@@ -1010,28 +978,24 @@ theme_drag_data_received_cb (GtkWidget *widget,
                              guint info, guint time,
                              AppearanceData *data)
 {
-  GList *uris;
+  gchar **uris;
 
   if (!(info == TARGET_URI_LIST || info == TARGET_NS_URL))
     return;
 
-  uris = gnome_vfs_uri_list_parse ((gchar *) selection_data->data);
+  uris = g_uri_list_extract_uris ((gchar *) selection_data->data);
 
-  if (uris != NULL && uris->data != NULL) {
-    GnomeVFSURI *uri = (GnomeVFSURI *) uris->data;
-    gchar *filename;
-
-    if (gnome_vfs_uri_is_local (uri))
-      filename = gnome_vfs_unescape_string (gnome_vfs_uri_get_path (uri), G_DIR_SEPARATOR_S);
-    else
-      filename = gnome_vfs_unescape_string (gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE), G_DIR_SEPARATOR_S);
-
-    gnome_vfs_uri_list_unref (uris);
+  if (uris != NULL && uris[0] != NULL) {
+    GFile *f = g_file_new_for_uri (uris[0]);
+    gchar *filename = g_file_get_path (f);
+    g_object_unref (f);
 
     gnome_theme_install_from_uri (filename,
         GTK_WINDOW (glade_xml_get_widget (data->xml, "appearance_window")));
     g_free (filename);
   }
+
+  g_strfreev (uris);
 }
 
 static void
@@ -1056,7 +1020,7 @@ themes_init (AppearanceData *data)
   GtkSettings *settings;
 
   /* initialise some stuff */
-  gnome_theme_init (NULL);
+  gnome_theme_init ();
   gnome_wm_manager_init ();
 
   data->revert_application_font = NULL;
