@@ -7,6 +7,7 @@
 #include <metacity-private/preview-widget.h>
 #include <signal.h>
 #include <errno.h>
+#include <math.h>
 
 /* We have to #undef this as metacity #defines these. */
 #undef _
@@ -19,8 +20,6 @@
 #include "gtkrc-utils.h"
 #include "capplet-util.h"
 
-static gint child_pid;
-
 typedef struct
 {
   gboolean set;
@@ -129,6 +128,39 @@ hbox_foreach (GtkWidget *widget,
   }
 }
 
+static void
+pixbuf_apply_mask_region (GdkPixbuf *pixbuf, GdkRegion *region)
+{
+  gint nchannels, rowstride, w, h;
+  guchar *pixels, *p;
+
+  g_return_if_fail (pixbuf);
+  g_return_if_fail (region);
+
+  nchannels = gdk_pixbuf_get_n_channels (pixbuf);
+  rowstride = gdk_pixbuf_get_rowstride (pixbuf);
+  pixels = gdk_pixbuf_get_pixels (pixbuf);
+
+
+  /* we need an alpha channel ... */
+  if (!gdk_pixbuf_get_has_alpha (pixbuf) || nchannels != 4)
+    return;
+
+  for (w = 0; w < gdk_pixbuf_get_width (pixbuf); ++w)
+    for (h = 0; h < gdk_pixbuf_get_height (pixbuf); ++h)
+    {
+      if (!gdk_region_point_in (region, w, h))
+      {
+        p = pixels + h * rowstride + w * nchannels;
+        if (G_BYTE_ORDER == G_BIG_ENDIAN)
+          p[0] = 0x0;
+        else
+          p[3] = 0x0;
+      }
+    }
+
+}
+
 static GdkPixbuf *
 create_folder_icon (char *icon_theme_name)
 {
@@ -226,6 +258,7 @@ create_meta_theme_pixbuf (ThemeThumbnailData *theme_thumbnail_data)
   MetaTheme *theme;
   GdkPixbuf *pixbuf, *icon;
   int icon_width, icon_height;
+  GdkRegion *region;
 
   g_object_set (gtk_settings_get_default (),
     "gtk-theme-name", (char *) theme_thumbnail_data->control_theme_name->data,
@@ -233,7 +266,10 @@ create_meta_theme_pixbuf (ThemeThumbnailData *theme_thumbnail_data)
     "gtk-icon-theme-name", (char *) theme_thumbnail_data->icon_theme_name->data,
     "gtk-color-scheme", (char *) theme_thumbnail_data->gtk_color_scheme->data,
     NULL);
+
   theme = meta_theme_load ((char *) theme_thumbnail_data->wm_theme_name->data, NULL);
+  if (theme == NULL)
+    return NULL;
 
   /* Represent the icon theme */
   icon = create_folder_icon ((char *) theme_thumbnail_data->icon_theme_name->data);
@@ -329,6 +365,11 @@ create_meta_theme_pixbuf (ThemeThumbnailData *theme_thumbnail_data)
                         vbox->allocation.x + vbox->allocation.width - icon_width - 5,
                         vbox->allocation.y + vbox->allocation.height - icon_height - 5,
                         1.0, 1.0, GDK_INTERP_BILINEAR, 255);
+  region = meta_preview_get_clip_region (META_PREVIEW (preview),
+      META_THUMBNAIL_SIZE, META_THUMBNAIL_SIZE);
+  pixbuf_apply_mask_region (pixbuf, region);
+  gdk_region_destroy (region);
+
   g_object_unref (icon);
   gtk_widget_destroy (window);
   meta_theme_free (theme);
@@ -430,8 +471,11 @@ create_metacity_theme_pixbuf (ThemeThumbnailData *theme_thumbnail_data)
   GdkVisual *visual;
   GdkPixmap *pixmap;
   GdkPixbuf *pixbuf, *retval;
+  GdkRegion *region;
 
   theme = meta_theme_load ((char *) theme_thumbnail_data->wm_theme_name->data, NULL);
+  if (theme == NULL)
+    return NULL;
 
   flags = META_FRAME_ALLOWS_DELETE |
           META_FRAME_ALLOWS_MENU |
@@ -486,6 +530,12 @@ create_metacity_theme_pixbuf (ThemeThumbnailData *theme_thumbnail_data)
   pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, (int) METACITY_THUMBNAIL_WIDTH * 1.2, (int) METACITY_THUMBNAIL_HEIGHT * 1.2);
   gdk_pixbuf_get_from_drawable (pixbuf, pixmap, NULL, 0, 0, 0, 0, (int) METACITY_THUMBNAIL_WIDTH * 1.2, (int) METACITY_THUMBNAIL_HEIGHT * 1.2);
 
+  region = meta_preview_get_clip_region (META_PREVIEW (preview),
+      METACITY_THUMBNAIL_WIDTH * 1.2, METACITY_THUMBNAIL_HEIGHT * 1.2);
+  pixbuf_apply_mask_region (pixbuf, region);
+  gdk_region_destroy (region);
+
+
   retval = gdk_pixbuf_scale_simple (pixbuf,
                                     METACITY_THUMBNAIL_WIDTH,
                                     METACITY_THUMBNAIL_HEIGHT,
@@ -665,10 +715,14 @@ message_from_capplet (GIOChannel   *source,
         else
           g_assert_not_reached ();
 
-        width = gdk_pixbuf_get_width (pixbuf);
-        height = gdk_pixbuf_get_height (pixbuf);
-        rowstride = gdk_pixbuf_get_rowstride (pixbuf);
-        pixels = gdk_pixbuf_get_pixels (pixbuf);
+        if (pixbuf == NULL) {
+          width = height = 0;
+        } else {
+          width = gdk_pixbuf_get_width (pixbuf);
+          height = gdk_pixbuf_get_height (pixbuf);
+          rowstride = gdk_pixbuf_get_rowstride (pixbuf);
+          pixels = gdk_pixbuf_get_pixels (pixbuf);
+        }
 
         /* Write the pixbuf's size */
         write (pipe_from_factory_fd[1], &width, sizeof (width));
@@ -676,16 +730,18 @@ message_from_capplet (GIOChannel   *source,
 
         for (i = 0; i < height; i++)
         {
-          write (pipe_from_factory_fd[1], pixels + (rowstride)*i, width * gdk_pixbuf_get_n_channels (pixbuf));
+          write (pipe_from_factory_fd[1], pixels + rowstride * i, width * gdk_pixbuf_get_n_channels (pixbuf));
         }
-        g_object_unref (pixbuf);
-        theme_thumbnail_data->status = READY_FOR_THEME;
+
+        if (pixbuf)
+          g_object_unref (pixbuf);
         g_byte_array_set_size (theme_thumbnail_data->type, 0);
         g_byte_array_set_size (theme_thumbnail_data->control_theme_name, 0);
         g_byte_array_set_size (theme_thumbnail_data->gtk_color_scheme, 0);
         g_byte_array_set_size (theme_thumbnail_data->wm_theme_name, 0);
         g_byte_array_set_size (theme_thumbnail_data->icon_theme_name, 0);
         g_byte_array_set_size (theme_thumbnail_data->application_font, 0);
+        theme_thumbnail_data->status = READY_FOR_THEME;
       }
       return TRUE;
 
@@ -750,6 +806,9 @@ message_from_child (GIOChannel   *source,
   if (async_data.set == FALSE)
     return TRUE;
 
+  if (condition == G_IO_HUP)
+    return FALSE;
+
   status = g_io_channel_read_chars (source,
                                     buffer,
                                     1024,
@@ -760,39 +819,46 @@ message_from_child (GIOChannel   *source,
     case G_IO_STATUS_NORMAL:
       g_byte_array_append (async_data.data, (guchar *) buffer, bytes_read);
 
-      if (async_data.thumbnail_width == 0 && async_data.data->len >= 2 * sizeof (gint))
+      if (async_data.thumbnail_width == -1 && async_data.data->len >= 2 * sizeof (gint))
       {
         async_data.thumbnail_width = *((gint *) async_data.data->data);
         async_data.thumbnail_height = *(((gint *) async_data.data->data) + 1);
         g_byte_array_remove_range (async_data.data, 0, 2 * sizeof (gint));
       }
-      else if (async_data.thumbnail_width > 0 && async_data.data->len == async_data.thumbnail_width * async_data.thumbnail_height * 4)
+
+      if (async_data.thumbnail_width >= 0 && async_data.data->len == async_data.thumbnail_width * async_data.thumbnail_height * 4)
       {
-        GdkPixbuf *pixbuf;
-        gchar *pixels;
-        gint i, rowstride;
+        GdkPixbuf *pixbuf = NULL;
 
-        pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, async_data.thumbnail_width, async_data.thumbnail_height);
-        pixels = (gchar *) gdk_pixbuf_get_pixels (pixbuf);
-        rowstride = gdk_pixbuf_get_rowstride (pixbuf);
+        if (async_data.thumbnail_width > 0) {
+          gchar *pixels;
+          gint i, rowstride;
 
-        for (i = 0; i < async_data.thumbnail_height; i++)
-          memcpy (pixels + rowstride * i, async_data.data->data + 4 * async_data.thumbnail_width * i, async_data.thumbnail_width * 4);
+          pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, async_data.thumbnail_width, async_data.thumbnail_height);
+          pixels = (gchar *) gdk_pixbuf_get_pixels (pixbuf);
+          rowstride = gdk_pixbuf_get_rowstride (pixbuf);
 
-        /* callback function needs to unref the pixbuf */
+          for (i = 0; i < async_data.thumbnail_height; ++i)
+            memcpy (pixels + rowstride * i, async_data.data->data + 4 * async_data.thumbnail_width * i, async_data.thumbnail_width * 4);
+        }
+
+        /* callback function needs to ref the pixbuf if it wants to keep it */
         (* async_data.func) (pixbuf, async_data.theme_name, async_data.user_data);
 
         if (async_data.destroy)
           (* async_data.destroy) (async_data.user_data);
 
+        if (pixbuf)
+          g_object_unref (pixbuf);
+
         /* Clean up async_data */
         g_free (async_data.theme_name);
         g_source_remove (async_data.watch_id);
         g_io_channel_unref (async_data.channel);
 
         /* reset async_data */
-        async_data.thumbnail_width = 0;
-        async_data.thumbnail_height = 0;
+        async_data.thumbnail_width = -1;
+        async_data.thumbnail_height = -1;
         async_data.theme_name = NULL;
         async_data.channel = NULL;
         async_data.func = NULL;
@@ -810,7 +876,7 @@ message_from_child (GIOChannel   *source,
 
     case G_IO_STATUS_EOF:
     case G_IO_STATUS_ERROR:
-      return TRUE;
+      return FALSE;
 
     default:
       g_assert_not_reached ();
@@ -868,10 +934,15 @@ read_pixbuf (void)
   do
   {
     bytes_read = read (pipe_from_factory_fd[0], ((guint8*) size) + j, 2 * sizeof (gint));
+    if (bytes_read == 0)
+      goto eof;
     j += bytes_read;
   }
   while (j < 2 * sizeof (gint));
 
+  if (size[0] <= 0 || size[1] <= 0)
+    return NULL;
+
   pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, size[0], size[1]);
   rowstride = gdk_pixbuf_get_rowstride (pixbuf);
   pixels = gdk_pixbuf_get_pixels (pixbuf);
@@ -882,25 +953,28 @@ read_pixbuf (void)
 
     do
     {
-      bytes_read = read (pipe_from_factory_fd[0], pixels + (rowstride)*i + j, size[0] * gdk_pixbuf_get_n_channels (pixbuf) - j);
+      bytes_read = read (pipe_from_factory_fd[0], pixels + rowstride * i + j, size[0] * gdk_pixbuf_get_n_channels (pixbuf) - j);
 
       if (bytes_read > 0)
         j += bytes_read;
       else if (bytes_read == 0)
       {
-        g_warning ("Received EOF while reading thumbnail");
         g_object_unref (pixbuf);
-        close (pipe_to_factory_fd[1]);
-        pipe_to_factory_fd[1] = 0;
-        close (pipe_from_factory_fd[0]);
-        pipe_from_factory_fd[0] = 0;
-        return NULL;
+        goto eof;
       }
     }
     while (j < size[0] * gdk_pixbuf_get_n_channels (pixbuf));
   }
 
   return pixbuf;
+
+eof:
+  g_warning ("Received EOF while reading thumbnail");
+  close (pipe_to_factory_fd[1]);
+  pipe_to_factory_fd[1] = 0;
+  close (pipe_from_factory_fd[0]);
+  pipe_from_factory_fd[0] = 0;
+  return NULL;
 }
 
 static GdkPixbuf *
@@ -1020,8 +1094,8 @@ generate_theme_thumbnail_async (gpointer            theme_info,
   }
 
   async_data.set = TRUE;
-  async_data.thumbnail_width = 0;
-  async_data.thumbnail_height = 0;
+  async_data.thumbnail_width = -1;
+  async_data.thumbnail_height = -1;
   async_data.theme_name = g_strdup (theme_name);
   async_data.func = func;
   async_data.user_data = user_data;
@@ -1111,9 +1185,19 @@ generate_icon_theme_thumbnail_async (GnomeThemeIconInfo *theme_info,
 void
 theme_thumbnail_factory_init (int argc, char *argv[])
 {
+#ifndef __APPLE__
+  gint child_pid;
+#endif
+
   pipe (pipe_to_factory_fd);
   pipe (pipe_from_factory_fd);
 
+/* Apple's CoreFoundation classes must not be used from forked
+ * processes. Since freetype (and thus GTK) uses them, we simply
+ * disable the thumbnailer on MacOS for now. That means no thumbs
+ * until the thumbnailing process is rewritten, but at least we won't
+ * make apps crash. */
+#ifndef __APPLE__
   child_pid = fork ();
   if (child_pid == 0)
   {
@@ -1152,10 +1236,9 @@ theme_thumbnail_factory_init (int argc, char *argv[])
   /* Parent */
   close (pipe_to_factory_fd[0]);
   close (pipe_from_factory_fd[1]);
+#endif /* __APPLE__ */
+
   async_data.set = FALSE;
-  async_data.thumbnail_width = 0;
-  async_data.thumbnail_height = 0;
   async_data.theme_name = NULL;
   async_data.data = g_byte_array_new ();
 }
-
