@@ -56,7 +56,8 @@
 #define USER_DOCS_STORE_FILE_NAME "documents.xbel"
 #define USER_DIRS_STORE_FILE_NAME "places.xbel"
 #define SYSTEM_STORE_FILE_NAME    "system-items.xbel"
-#define CALC_TEMPLATE_FILE_NAME   "empty.ods"
+#define CALC_TEMPLATE_FILE_NAME   "empty.ots"
+#define WRITER_TEMPLATE_FILE_NAME "empty.ott"
 
 #define GTK_BOOKMARKS_FILE ".gtk-bookmarks"
 
@@ -121,9 +122,7 @@ static void set_rank     (BookmarkAgent *, const gchar *, gint);
 
 static void load_xbel_store          (BookmarkAgent *);
 static void load_places_store        (BookmarkAgent *);
-static void load_recent_store        (BookmarkAgent *);
 static void update_user_spec_path    (BookmarkAgent *);
-static void update_recent_store_path (BookmarkAgent *);
 static void save_xbel_store          (BookmarkAgent *);
 static void create_app_item          (BookmarkAgent *, const gchar *);
 static void create_doc_item          (BookmarkAgent *, const gchar *);
@@ -299,6 +298,8 @@ bookmark_agent_remove_item (BookmarkAgent *this, const gchar *uri)
 				if (rank_i > rank)
 					set_rank (this, uris [i], rank_i - 1);
 			}
+
+			g_strfreev (uris);
 		}
 
 		save_store (this);
@@ -321,6 +322,85 @@ bookmark_agent_reorder_items (BookmarkAgent *this, const gchar **uris)
 	save_store (this);
 }
 
+static GList *
+make_items_from_bookmark_file (BookmarkAgent *this, GBookmarkFile *store)
+{
+	BookmarkAgentPrivate *priv = PRIVATE (this);
+	gchar **uris;
+	gint i;
+	GList *items_ordered;
+
+	if (!store)
+		return NULL;
+
+	uris = g_bookmark_file_get_uris (store, NULL);
+	items_ordered = NULL;
+
+	for (i = 0; uris && uris [i]; ++i) {
+		gboolean include;
+
+		if (priv->type == BOOKMARK_STORE_RECENT_APPS)
+			include = g_bookmark_file_has_group (store, uris [i], "recently-used-apps", NULL);
+		else
+			include = ! g_bookmark_file_get_is_private (store, uris [i], NULL);
+
+		if (include) {
+			BookmarkItem *item;
+
+			item = g_new0 (BookmarkItem, 1);
+
+			item->uri       = g_strdup (uris [i]);
+			item->mime_type = g_bookmark_file_get_mime_type (store, uris [i], NULL);
+			item->mtime     = g_bookmark_file_get_modified  (store, uris [i], NULL);
+
+			items_ordered = g_list_prepend (items_ordered, item);
+		}
+	}
+
+	items_ordered = g_list_sort (items_ordered, recent_item_mru_comp_func);
+
+	g_strfreev (uris);
+
+	return items_ordered;
+}
+
+void
+bookmark_agent_update_from_bookmark_file (BookmarkAgent *this, GBookmarkFile *store)
+{
+	BookmarkAgentPrivate *priv;
+	GList *items_ordered;
+	GList  *node;
+
+	g_return_if_fail (IS_BOOKMARK_AGENT (this));
+
+	priv = PRIVATE (this);
+
+	libslab_checkpoint ("bookmark_agent_update_from_bookmark_file(): start updating");
+
+	items_ordered = make_items_from_bookmark_file (this, store);
+
+	g_bookmark_file_free (priv->store);
+	priv->store = g_bookmark_file_new ();
+
+	for (node = items_ordered; node; node = node->next) {
+		BookmarkItem *item;
+
+		item = (BookmarkItem *) node->data;
+
+		g_bookmark_file_set_mime_type (priv->store, item->uri, item->mime_type);
+		g_bookmark_file_set_modified  (priv->store, item->uri, item->mtime);
+
+		bookmark_item_free (item);
+	}
+
+	g_list_free (items_ordered);
+
+	libslab_checkpoint ("bookmark_agent_update_from_bookmark_file(): updating internal items");
+	update_items (this);
+
+	libslab_checkpoint ("bookmark_agent_update_from_bookmark_file(): end updating");
+}
+
 void
 bookmark_item_free (BookmarkItem *item)
 {
@@ -465,13 +545,6 @@ bookmark_agent_new (BookmarkStoreType type)
 			priv->store_path = g_build_filename (g_get_home_dir (), ".recently-used", NULL);
 #endif
 
-			gnome_vfs_monitor_add (
-				& priv->store_monitor, priv->store_path,
-				GNOME_VFS_MONITOR_FILE, store_monitor_cb, this);
-
-			priv->update_path = update_recent_store_path;
-			priv->load_store  = load_recent_store;
-
 			break;
 
 		case BOOKMARK_STORE_SYSTEM:
@@ -572,8 +645,11 @@ update_agent (BookmarkAgent *this)
 {
 	BookmarkAgentPrivate *priv = PRIVATE (this);
 
-	priv->update_path (this);
-	priv->load_store  (this);
+	if (priv->update_path)
+		priv->update_path (this);
+
+	if (priv->load_store)
+		priv->load_store (this);
 
 	update_items (this);
 }
@@ -590,6 +666,7 @@ update_items (BookmarkAgent *this)
 	gint       rank_corr       = -1;
 	gboolean   needs_update    = FALSE;
 	gboolean   store_corrupted = FALSE;
+	gchar     *new_title, *old_title;
 
 	gint i;
 
@@ -625,9 +702,23 @@ update_items (BookmarkAgent *this)
 	if (priv->n_items != n_uris)
 		needs_update = TRUE;
 
-	for (i = 0; ! needs_update && uris_ordered && uris_ordered [i]; ++i)
-		if (strcmp (priv->items [i]->uri, uris_ordered [i]))
+	for (i = 0; ! needs_update && uris_ordered && uris_ordered [i]; ++i) {
+		if (priv->type == BOOKMARK_STORE_USER_DIRS) {
+			new_title = g_bookmark_file_get_title (priv->store, uris_ordered [i], NULL);
+			old_title = priv->items [i]->title;
+			if (!new_title && !old_title) {
+				if (strcmp (priv->items [i]->uri, uris_ordered [i]))
+					needs_update = TRUE;
+			}
+			else if ((new_title && !old_title) || (!new_title && old_title))
+				needs_update = TRUE;
+			else if (strcmp (old_title, new_title))
+				needs_update = TRUE;
+			g_free (new_title);
+		}
+		else if (strcmp (priv->items [i]->uri, uris_ordered [i]))
 			needs_update = TRUE;
+	}
 
 	if (needs_update) {
 		for (i = 0; priv->items && priv->items [i]; ++i)
@@ -650,7 +741,11 @@ update_items (BookmarkAgent *this)
 			g_bookmark_file_get_icon (priv->store, uris_ordered [i], & priv->items [i]->icon, NULL, NULL);
 		}
 
-		g_object_notify (G_OBJECT (this), BOOKMARK_AGENT_ITEMS_PROP);
+		/* Since the bookmark store for recently-used items is updated by the caller of BookmarkAgent,
+		 * we don't emit notifications in that case.  The caller will know when to update itself.
+		 */
+		if (!TYPE_IS_RECENT (priv->type))
+			g_object_notify (G_OBJECT (this), BOOKMARK_AGENT_ITEMS_PROP);
 	}
 
 	if (store_corrupted)
@@ -751,9 +846,17 @@ load_xbel_store (BookmarkAgent *this)
 	GError *error = NULL;
 
 	gint i;
+	gboolean success;
 
+	if (!priv->store_path)
+		success = FALSE;
+	else {
+		libslab_checkpoint ("load_xbel_store(): start loading %s", priv->store_path);
+		success = g_bookmark_file_load_from_file (priv->store, priv->store_path, & error);
+		libslab_checkpoint ("load_xbel_store(): end loading %s", priv->store_path);
+	}
 
-	if (! (priv->store_path && g_bookmark_file_load_from_file (priv->store, priv->store_path, & error))) {
+	if (!success) {
 		g_bookmark_file_free (priv->store);
 		priv->store = g_bookmark_file_new ();
 
@@ -764,10 +867,16 @@ load_xbel_store (BookmarkAgent *this)
 		return;
 	}
 
+	libslab_checkpoint ("load_xbel_store(): start creating items from %s", priv->store_path);
+
 	uris = g_bookmark_file_get_uris (priv->store, NULL);
 
 	for (i = 0; uris && uris [i]; ++i)
 		priv->create_item (this, uris [i]);
+
+	g_strfreev (uris);
+
+	libslab_checkpoint ("load_xbel_store(): end creating items from %s", priv->store_path);
 }
 
 static void
@@ -777,12 +886,11 @@ load_places_store (BookmarkAgent *this)
 
 	gchar **uris;
 	gchar **groups;
-	gchar **folders = NULL;
-
-	gchar  *buf;
-
-	gint i, j;
+	gchar **bookmarks = NULL;
+	
+	gchar  *buf, *label, *uri;
 
+	gint i, j, bookmark_len;
 
 	load_xbel_store (this);
 
@@ -807,74 +915,30 @@ load_places_store (BookmarkAgent *this)
 	g_file_get_contents (priv->gtk_store_path, & buf, NULL, NULL);
 
 	if (buf) {
-		folders = g_strsplit (buf, "\n", -1);
+		bookmarks = g_strsplit (buf, "\n", -1);
 		g_free (buf);
 	}
 
-	for (i = 0; folders && folders [i]; ++i) {
-		if (strlen (folders [i]) > 0) {
-			g_bookmark_file_add_group (priv->store, folders [i], "gtk-bookmarks");
-			priv->create_item (this, folders [i]);
+	for (i = 0; bookmarks && bookmarks [i]; ++i) {
+		bookmark_len = strlen (bookmarks [i]);
+		if (bookmark_len > 0) {
+			label = strstr (bookmarks[i], " ");
+			if (label != NULL)
+				uri = g_strndup (bookmarks [i], bookmark_len - strlen (label));
+			else
+				uri = bookmarks [i];
+			g_bookmark_file_add_group (priv->store, uri, "gtk-bookmarks");
+			priv->create_item (this, uri);
+			if (label != NULL) {
+				label++;
+				if (strlen (label) > 0)
+					g_bookmark_file_set_title (priv->store, uri, label);
+				g_free (uri);
+			}
 		}
 	}
 
-	g_strfreev (folders);
-}
-
-static void
-load_recent_store (BookmarkAgent *this)
-{
-	BookmarkAgentPrivate *priv = PRIVATE (this);
-
-	GBookmarkFile *store;
-
-	gchar **uris          = NULL;
-	GList  *items_ordered = NULL;
-
-	BookmarkItem *item;
-
-	gboolean include;
-
-	gint    i;
-	GList  *node;
-
-
-	store = g_bookmark_file_new ();
-	g_bookmark_file_load_from_file (store, priv->store_path, NULL);
-
-	uris = g_bookmark_file_get_uris (store, NULL);
-
-	for (i = 0; uris && uris [i]; ++i) {
-		if (priv->type == BOOKMARK_STORE_RECENT_APPS)
-			include = g_bookmark_file_has_group (store, uris [i], "recently-used-apps", NULL);
-		else
-			include = ! g_bookmark_file_get_is_private (store, uris [i], NULL);
-
-		if (include) {
-			item = g_new0 (BookmarkItem, 1);
-
-			item->uri       = g_strdup (uris [i]);
-			item->mime_type = g_bookmark_file_get_mime_type (store, uris [i], NULL);
-			item->mtime     = g_bookmark_file_get_modified  (store, uris [i], NULL);
-
-			items_ordered = g_list_insert_sorted (items_ordered, item, recent_item_mru_comp_func);
-		}
-	}
-
-	g_strfreev (uris);
-	g_bookmark_file_free (store);
-
-	g_bookmark_file_free (priv->store);
-	priv->store = g_bookmark_file_new ();
-
-	for (node = items_ordered; node; node = node->next) {
-		item = (BookmarkItem *) node->data;
-
-		g_bookmark_file_set_mime_type (priv->store, item->uri, item->mime_type);
-		g_bookmark_file_set_modified  (priv->store, item->uri, item->mtime);
-
-		bookmark_item_free (item);
-	}
+	g_strfreev (bookmarks);
 }
 
 static gchar *
@@ -957,25 +1021,6 @@ update_user_spec_path (BookmarkAgent *this)
 		g_free (path);
 }
 
-static void
-update_recent_store_path (BookmarkAgent *this)
-{
-	BookmarkAgentPrivate *priv = PRIVATE (this);
-
-	BookmarkStoreStatus status;
-
-
-	if (g_file_test (priv->store_path, G_FILE_TEST_EXISTS))
-		status = BOOKMARK_STORE_USER;
-	else
-		status = BOOKMARK_STORE_USER;
-
-	if (priv->status != status) {
-		priv->status = status;
-		g_object_notify (G_OBJECT (this), BOOKMARK_AGENT_STORE_STATUS_PROP);
-	}
-}
-
 static void
 save_xbel_store (BookmarkAgent *this)
 {
@@ -996,6 +1041,7 @@ create_app_item (BookmarkAgent *this, const gchar *uri)
 
 	GnomeDesktopItem *ditem;
 	gchar *uri_new = NULL;
+	gchar *name;
 
 
 	ditem = libslab_gnome_desktop_item_new_from_unknown_id (uri);
@@ -1008,9 +1054,18 @@ create_app_item (BookmarkAgent *this, const gchar *uri)
 	if (! uri_new)
 		return;
 
-	if (libslab_strcmp (uri, uri_new)) {
+	name = g_bookmark_file_get_title (priv->store, uri, NULL);
+	if (! libslab_strcmp (name, "Help"))
+		g_bookmark_file_set_title (priv->store, uri, _("Help"));
+	else if (! libslab_strcmp (name, "Lock Screen"))
+		g_bookmark_file_set_title (priv->store, uri, _("Lock Screen"));
+	else if (! libslab_strcmp (name, "Logout"))
+		g_bookmark_file_set_title (priv->store, uri, _("Logout"));
+	else if (! libslab_strcmp (name, "Shutdown"))
+		g_bookmark_file_set_title (priv->store, uri, _("Shutdown"));
+
+	if (libslab_strcmp (uri, uri_new))
 		g_bookmark_file_move_item (priv->store, uri, uri_new, NULL);
-	}
 }
 
 static void
@@ -1031,11 +1086,15 @@ create_doc_item (BookmarkAgent *this, const gchar *uri)
 		dir = g_build_filename (g_get_home_dir (), "Documents", NULL);
 
 		if (! strcmp (uri, "BLANK_SPREADSHEET")) {
-			file = g_strconcat (_("New Spreadsheet"), ".ods", NULL);
+			g_bookmark_file_set_title (priv->store, uri, "BLANK_SPREADSHEET");
+			file = g_strconcat (_("New Spreadsheet"), ".ots", NULL);
 			template = find_package_data_file (CALC_TEMPLATE_FILE_NAME);
 		}
-		else
-			file = g_strconcat (_("New Document"), ".odt", NULL);
+		else {
+			g_bookmark_file_set_title (priv->store, uri, "BLANK_DOCUMENT");
+			file = g_strconcat (_("New Document"), ".ott", NULL);
+			template = find_package_data_file (WRITER_TEMPLATE_FILE_NAME);
+		}
 
 		path = g_build_filename (dir, file, NULL);
 
@@ -1047,7 +1106,6 @@ create_doc_item (BookmarkAgent *this, const gchar *uri)
 					g_file_set_contents (path, contents, length, NULL);
 
 				g_free (contents);
-				g_free (template);
 			}
 			else
 				fclose (g_fopen (path, "w"));
@@ -1058,6 +1116,7 @@ create_doc_item (BookmarkAgent *this, const gchar *uri)
 		g_free (dir);
 		g_free (file);
 		g_free (path);
+		g_free (template);
 	}
 
 	if (! uri_new)
@@ -1090,11 +1149,13 @@ create_dir_item (BookmarkAgent *this, const gchar *uri)
 	}
 	else if (! strcmp (uri, "DOCUMENTS")) {
 		path = g_build_filename (g_get_home_dir (), "Documents", NULL);
+		name = _("Documents");
 		uri_new = g_filename_to_uri (path, NULL, NULL);
 		g_free (path);
 	}
 	else if (! strcmp (uri, "DESKTOP")) {
 		path = g_build_filename (g_get_home_dir (), "Desktop", NULL);
+		name = _("Desktop");
 		uri_new = g_filename_to_uri (path, NULL, NULL);
 		icon = "gnome-fs-desktop";
 		g_free (path);
